const { sum } = require("lodash");

var roleHarvester = {

    minimum_parts: [WORK, MOVE, CARRY],
    recommended_parts: [WORK, MOVE, CARRY],
    min_cost: sum([BODYPART_COST["work"], BODYPART_COST["move"], BODYPART_COST["carry"]]),

    /**
     * Orders the given spawn to create a creep fitting for this role according to the recommended_parts and if impossible use minimum_parts instead
     * @param {spawn} spawn
     * @param {string} postfix string to postfix to the creep
     * @param {string} owner_id strind id of who shall be entered as owner of the new creep
     * 
     * @return {string} name of the new creep
     */
    spawn: function(spawn, postfix, owner_id) {
        if (spawn.spawnCreep(this.recommended_parts, "harvester-"+postfix, {dryRun: true}) === OK) {
            console.log("roleHarvester: Spawn "+spawn.name+" is creating harvester-"+postfix+" with recommended parts ...");
            spawn.spawnCreep(this.recommended_parts, "harvester-"+postfix, {memory: {"owner": owner_id, "role": "Harv"}});
        }
        else if (spawn.spawnCreep(this.minimum_parts, "harvester-"+postfix, {dryRun: true}) === OK) {
            console.log("roleHarvester: Spawn "+spawn.name+" is creating harvester-"+postfix+" with minimum parts ...");
            spawn.spawnCreep(this.minimum_parts, "harvester-"+postfix, {memory: {"owner": owner_id, "role": "Harv"}});
        }
        else {
            console.log("roleHarvester: Spawn \""+spawn.name+"\" couldn't afford a harvester! D: Result: ", spawn.spawnCreep(this.minimum_parts, "harvester-"+postfix, {dryRun: true}));
            throw "Not enough energy!";
        }

        return "harvester-"+postfix;
    },

    /**
     * Runs the logic and actions of the harvester
     * @param {Creep} creep 
     */
    run: function(creep) {
        if (creep.id === undefined) {  // id is undefined if the creep was created this tick
            return
        }
        console.log("roleHarvester ["+creep.name+"]: run() " + creep + " id:", creep.id);

        // TODO implement state machine here about what the harvester is up to. Currently it only moves to the nearest source.

        let goals = _.map(creep.room.find(FIND_SOURCES), function(source) {
            return {pos: source.pos, range: 1};
        });

        let nearest_source = PathFinder.search(creep.pos, goals);
        console.log("roleHarvester ["+creep.name+"]: run() moving creept towards " + nearest_source.path[nearest_source.path.length-1], nearest_source);
        
        console.log("roleHarvester ["+creep.name+"]: run() moving to direction " + nearest_source.path[0]);
        creep.moveTo(nearest_source.path[0]);
        
    },
};

module.exports = roleHarvester;